@@ -51,7 +51,7 @@ def get_all_columns(project):
     data_types.update(project.data_types.items())
     # all data types from import data
     if project.derived_all_input_schema:
-        data_types.update({key: 'Unknown' for key in project.derived_all_input_schema})
+        data_types.update({key: 'Unknown' for key in project.derived_all_input_schema if key not in data_types})
 
     # remove $undefined$ if there is one type at least in labeling config, because it will be resolved automatically
     if len(project.data_types) > 0:
@@ -327,7 +327,7 @@ def load_task(project, task_id, params, resolve_uri=False):
         task['predictions_results'] = json.dumps([item.get('result', []) for item in predictions])
         scores = [p['score'] for p in predictions if 'score' in p]
         if scores:
-            task['prediction_scores'] = sum(scores) / len(scores)
+            task['predictions_score'] = sum(scores) / len(scores)
     else:
         task['predictions_results'] = ''
 
