@@ -1,11 +1,41 @@
-import json
+import ujson as json
 import os
 import logging
 from copy import deepcopy
 
-from label_studio.utils.io import json_load, delete_dir_content, iter_files
+from label_studio.utils.io import json_load, delete_dir_content, iter_files, remove_file_or_dir
 from .base import BaseStorage, BaseForm, CloudStorage
 
+import collections
+
+
+class ReadOnlyDict(collections.Mapping):
+
+    def __init__(self, data):
+        self._data = data
+        self._id = None
+
+    def __setitem__(self, key, value):
+        if key == 'id':
+            self._id = value
+        else:
+            raise ValueError("Can't change read-only only dict: " + key + ' = ' + str(value))
+
+    def __getitem__(self, key):
+        if key == 'id':
+            return self._id
+        return self._data[key]
+
+    def __len__(self):
+        return len(self._data)
+
+    def __iter__(self):
+        return iter(self._data)
+
+    @property
+    def __class__(self):
+        return dict
+
 
 logger = logging.getLogger(__name__)
 
@@ -63,8 +93,11 @@ class JSONStorage(BaseStorage):
         self.data.pop(int(key), None)
         self._save()
 
-    def remove_all(self):
-        self.data = {}
+    def remove_all(self, ids=None):
+        if ids is None:
+            self.data = {}
+        else:
+            [self.data.pop(i, None) for i in ids]
         self._save()
 
     def empty(self):
@@ -86,15 +119,21 @@ class DirJSONsStorage(BaseStorage):
     def __init__(self, **kwargs):
         super(DirJSONsStorage, self).__init__(**kwargs)
         os.makedirs(self.path, exist_ok=True)
+        self.cache = {}
 
     @property
     def readable_path(self):
         return self.path
 
     def get(self, id):
-        filename = os.path.join(self.path, str(id) + '.json')
-        if os.path.exists(filename):
-            return json_load(filename)
+        if id in self.cache:
+            return self.cache[id]
+        else:
+            filename = os.path.join(self.path, str(id) + '.json')
+            if os.path.exists(filename):
+                data = json_load(filename)
+                self.cache[id] = data
+                return data
 
     def __contains__(self, id):
         return id in set(self.ids())
@@ -103,8 +142,10 @@ class DirJSONsStorage(BaseStorage):
         filename = os.path.join(self.path, str(id) + '.json')
         with open(filename, 'w', encoding='utf8') as fout:
             json.dump(value, fout, indent=2, sort_keys=True)
+        self.cache[id] = value
 
     def set_many(self, keys, values):
+        self.cache.clear()
         raise NotImplementedError
 
     def ids(self):
@@ -118,17 +159,28 @@ class DirJSONsStorage(BaseStorage):
         pass
 
     def items(self):
-        for key in self.ids():
-            filename = os.path.join(self.path, str(key) + '.json')
-            yield key, json_load(filename)
+        for id in self.ids():
+            filename = os.path.join(self.path, str(id) + '.json')
+            yield id, self.cache[id] if id in self.cache else json_load(filename)
 
-    def remove(self, key):
-        filename = os.path.join(self.path, str(key) + '.json')
+    def remove(self, id):
+        filename = os.path.join(self.path, str(id) + '.json')
         if os.path.exists(filename):
             os.remove(filename)
+            self.cache.pop(id, None)
 
-    def remove_all(self):
-        delete_dir_content(self.path)
+    def remove_all(self, ids=None):
+        if ids is None:
+            self.cache.clear()
+            delete_dir_content(self.path)
+        else:
+            for i in ids:
+                self.cache.pop(i, None)
+                path = os.path.join(self.path, str(i) + '.json')
+                try:
+                    remove_file_or_dir(path)
+                except OSError:
+                    logger.warning('Storage file already removed: ' + path)
 
     def empty(self):
         return next(self.ids(), None) is None
@@ -183,21 +235,23 @@ class ExternalTasksJSONStorage(CloudStorage):
     def readable_path(self):
         return self.path
 
-    def _get_value(self, key):
-        return deepcopy(self.data[int(key)])
+    def _get_value(self, key, inplace=False):
+        return self.data[int(key)] if inplace else deepcopy(self.data[int(key)])
 
     def _set_value(self, key, value):
         self.data[int(key)] = value
 
     def set(self, id, value):
-        super(ExternalTasksJSONStorage, self).set(id, value)
-        self._save()
+        with self.thread_lock:
+            super(ExternalTasksJSONStorage, self).set(id, value)
+            self._save()
 
     def set_many(self, ids, values):
-        for id, value in zip(ids, values):
-            super(ExternalTasksJSONStorage, self)._pre_set(id, value)
-        self._save_ids()
-        self._save()
+        with self.thread_lock:
+            for id, value in zip(ids, values):
+                super(ExternalTasksJSONStorage, self)._pre_set(id, value)
+            self._save_ids()
+            self._save()
 
     def _extract_task_id(self, full_key):
         return int(full_key.split(self.key_prefix, 1)[-1])
@@ -211,33 +265,41 @@ class ExternalTasksJSONStorage(CloudStorage):
 
     def _remove_id_from_keys_map(self, id):
         full_key = self.key_prefix + str(id)
+        assert id in self._ids_keys_map, 'No such task id: ' + str(id)
         assert self._ids_keys_map[id]['key'] == full_key, (self._ids_keys_map[id]['key'], full_key)
         self._selected_ids.remove(id)
         self._ids_keys_map.pop(id)
         self._keys_ids_map.pop(full_key)
 
     def remove(self, id):
-        id = int(id)
+        with self.thread_lock:
+            id = int(id)
 
-        logger.debug('Remove id=' + str(id) + ' from ids.json')
-        self._remove_id_from_keys_map(id)
-        self._save_ids()
-
-        logger.debug('Remove id=' + str(id) + ' from tasks.json')
-        self.data.pop(id, None)
-        self._save()
-
-    def remove_all(self):
-
-        logger.debug('Remove ' + str(len(self.data)) + ' records from ids.json')
-        for id in self.data:
+            logger.debug('Remove id=' + str(id) + ' from ids.json')
             self._remove_id_from_keys_map(id)
-        self._save_ids()
+            self._save_ids()
 
-        logger.debug('Remove all data from tasks.json')
-        # remove record from tasks.json
-        self.data = {}
-        self._save()
+            logger.debug('Remove id=' + str(id) + ' from tasks.json')
+            self.data.pop(id, None)
+            self._save()
+
+    def remove_all(self, ids=None):
+        with self.thread_lock:
+            remove_ids = self.data if ids is None else ids
+
+            logger.debug('Remove ' + str(len(remove_ids)) + ' records from ids.json')
+            for id in remove_ids:
+                self._remove_id_from_keys_map(id)
+            self._save_ids()
+
+            logger.debug('Remove all data from tasks.json')
+            # remove record from tasks.json
+            if ids is None:
+                self.data = {}
+            else:
+                for id in remove_ids:
+                    self.data.pop(id, None)
+            self._save()
 
 
 class CompletionsDirStorage(DirJSONsStorage):
